'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _expect = require('expect');

var _expect2 = _interopRequireDefault(_expect);

var _reduxSagaTestPlan = require('redux-saga-test-plan');

var _effects = require('redux-saga/effects');

var _providers = require('redux-saga-test-plan/providers');

var _reduxOfflineStatus = require('redux-offline-status');

var _request = require('./request.sagas');

var _request2 = require('./request.actions');

var _requestQueue = require('../requestQueue/requestQueue.actions');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('waitForOnline saga', function () {
  it('does not wait for ONLINE if already online', function () {
    return (0, _reduxSagaTestPlan.expectSaga)(_request.waitForOnline).withState({ offline: false }).not.call.fn((0, _effects.take)(_reduxOfflineStatus.ONLINE)).run();
  });

  it('returns true when the application is online', function () {
    return (0, _reduxSagaTestPlan.expectSaga)(_request.waitForOnline).withState({ offline: false }).returns(true).run();
  });

  it('waits for the application to come online when offline', function () {
    return (0, _reduxSagaTestPlan.expectSaga)(_request.waitForOnline).withState({ offline: true }).take(_reduxOfflineStatus.ONLINE).silentRun();
  });

  it('when offline, it returns when an ONLINE action is fired', function () {
    return (0, _reduxSagaTestPlan.expectSaga)(_request.waitForOnline).withState({ offline: true }).delay(250).dispatch({ type: _reduxOfflineStatus.ONLINE }).run(500);
  });
});

describe('getRequest Saga', function () {
  var res = {
    data: 'test'
  };

  var apiCall = function apiCall(param1, param2) {
    return Promise.resolve(res);
  };

  var action = {
    type: 'TEST',
    params: { testParam: 'stuff' }
  };

  it('it pauses the saga when offline is true', function () {
    return (0, _reduxSagaTestPlan.expectSaga)(_request.getRequest, apiCall, action).withState({ offline: true }).call(_request.waitForOnline).silentRun();
  });

  it('makes API request if ONLINE', function () {
    return (0, _reduxSagaTestPlan.expectSaga)(_request.getRequest, apiCall, action).withState({ offline: false }).call(apiCall, action.params).run();
  });

  it('emits a _SUCCESS with the response action when the API request returns a response', function () {
    return (0, _reduxSagaTestPlan.expectSaga)(_request.getRequest, apiCall, action).withState({ offline: false }).provide([[(0, _effects.call)(apiCall, action.params), res]]).put((0, _request2.requestSuccess)(action.type, res)).run();
  });

  it('returns the API response when done', function () {
    return (0, _reduxSagaTestPlan.expectSaga)(_request.getRequest, apiCall, action).withState({ offline: false }).provide([[(0, _effects.call)(apiCall, action.params), res]]).returns(res).run();
  });

  it('emits an OFFLINE action if there is a Network Error', function () {
    var error = new Error('Network Error');

    return (0, _reduxSagaTestPlan.expectSaga)(_request.getRequest, apiCall, action).withState({ offline: false }).provide([[(0, _effects.call)(apiCall, action.params), (0, _providers.throwError)(error)], [(0, _effects.call)(_request.getRequest, apiCall, action)]]).put({ type: _reduxOfflineStatus.OFFLINE }).run();
  });

  it('restarts the saga if there is a Network Error', function () {
    var error = new Error('Network Error');

    return (0, _reduxSagaTestPlan.expectSaga)(_request.getRequest, apiCall, action).withState({ offline: false }).provide([[(0, _effects.call)(apiCall, action.params), (0, _providers.throwError)(error)], [(0, _effects.call)(_request.getRequest, apiCall, action)]]).call(_request.getRequest, apiCall, action).run();
  });

  it('emits a _ERROR action with an error when the API request throws an error', function () {
    var error = new Error('There was an error');

    return (0, _reduxSagaTestPlan.expectSaga)(_request.getRequest, apiCall, action).withState({ offline: false }).provide([[(0, _effects.call)(apiCall, action.params), (0, _providers.throwError)(error)]]).put((0, _request2.requestError)(action.type, error)).run();
  });

  it('returns an error when the API request throws a non-network error', function () {
    var error = new Error('There was an error');

    return (0, _reduxSagaTestPlan.expectSaga)(_request.getRequest, apiCall, action).withState({ offline: false }).provide([[(0, _effects.call)(apiCall, action.params), (0, _providers.throwError)(error)]]).run(function (result) {
      return (0, _expect2.default)(result.returnValue).toEqual(error);
    });
  });
});

describe('fetchQueuedRequest Saga', function () {
  var action = {
    type: 'TEST_QUEUE',
    id: '1234',
    url: 'https://google.com',
    payload: undefined,
    method: 'POST'
  };

  var res = {
    data: true
  };

  it('pauses the saga when offline', function () {
    return (0, _reduxSagaTestPlan.expectSaga)(_request.fetchQueuedRequest, action).withState({ offline: true }).call(_request.waitForOnline).silentRun();
  });

  it('when paused, it continues the saga when an ONLINE action is emitted', function () {
    return (0, _reduxSagaTestPlan.expectSaga)(_request.fetchQueuedRequest, action).withState({ offline: true }).provide([[(0, _effects.call)(_request.fetchJson, action.url, action.method, action.payload), res]]).delay(200).dispatch({ type: _reduxOfflineStatus.ONLINE }).call(_request.fetchJson, action.url, action.method, action.payload).run();
  });

  it('makes API request if ONLINE', function () {
    return (0, _reduxSagaTestPlan.expectSaga)(_request.fetchQueuedRequest, action).withState({ offline: false }).provide([[(0, _effects.call)(_request.fetchJson, action.url, action.method, action.payload), res]]).call(_request.fetchJson, action.url, action.method, action.payload).run();
  });

  it('emits a queued _SUCCESS with the response action when the API request returns a response', function () {
    return (0, _reduxSagaTestPlan.expectSaga)(_request.fetchQueuedRequest, action).withState({ offline: false }).provide([[(0, _effects.call)(_request.fetchJson, action.url, action.method, action.payload), res]]).put((0, _request2.requestSuccess)(action.type, _extends({}, res, {
      queued: true,
      parent: action
    }))).run();
  });

  it('dispatches a DEQUEUE_REQUEST action on request SUCCESS', function () {
    return (0, _reduxSagaTestPlan.expectSaga)(_request.fetchQueuedRequest, action).withState({ offline: false }).provide([[(0, _effects.call)(_request.fetchJson, action.url, action.method, action.payload), res]]).put((0, _requestQueue.dequeueRequest)(action.id)).run();
  });

  it('returns the API response when done', function () {
    return (0, _reduxSagaTestPlan.expectSaga)(_request.fetchQueuedRequest, action).withState({ offline: false }).provide([[(0, _effects.call)(_request.fetchJson, action.url, action.method, action.payload), res]]).returns(res).run();
  });

  it('dispatches an OFFLINE action if there is a Network Error', function () {
    var error = new Error('Network Error');

    return (0, _reduxSagaTestPlan.expectSaga)(_request.fetchQueuedRequest, action).withState({ offline: false }).provide([[(0, _effects.call)(_request.fetchJson, action.url, action.method, action.payload), (0, _providers.throwError)(error)]]).put({ type: _reduxOfflineStatus.OFFLINE }).run();
  });

  it('does not dispatch a DEQUEUE_REQUEST action on Network Error', function () {
    var error = new Error('Network Error');

    return (0, _reduxSagaTestPlan.expectSaga)(_request.fetchQueuedRequest, action).withState({ offline: false }).provide([[(0, _effects.call)(_request.fetchJson, action.url, action.method, action.payload), (0, _providers.throwError)(error)]]).not.call.fn((0, _requestQueue.dequeueRequest)(action.id)).run();
  });

  it('dispatches a _ERROR action with an error when the API request throws an error', function () {
    var error = new Error('There was an error');

    return (0, _reduxSagaTestPlan.expectSaga)(_request.fetchQueuedRequest, action).withState({ offline: false }).provide([[(0, _effects.call)(_request.fetchJson, action.url, action.method, action.payload), (0, _providers.throwError)(error)]]).put((0, _request2.requestError)(action.type, error)).run();
  });

  it('returns an error when when the request returns an error', function () {
    var error = new Error('There was an error');

    return (0, _reduxSagaTestPlan.expectSaga)(_request.fetchQueuedRequest, action).withState({ offline: false }).provide([[(0, _effects.call)(_request.fetchJson, action.url, action.method, action.payload), (0, _providers.throwError)(error)]]).run(function (result) {
      return (0, _expect2.default)(result.returnValue).toEqual(error);
    });
  });
});